@@ -67,3 +67,38 @@ def test_extra_met(platform):
     with TempVirtualEnv(sys.executable, platform) as venv:
         platform.run_command(['pip', 'install', 'requests[security]==2.25.1'], check=True, capture_output=True)
         assert dependencies_in_sync([Requirement('requests[security]==2.25.1')], venv.sys_path)
+
+
+@pytest.mark.requires_internet
+@pytest.mark.requires_git
+def test_dependency_git(platform):
+    with TempVirtualEnv(sys.executable, platform) as venv:
+        platform.run_command(
+            ['pip', 'install', 'requests@git+https://github.com/psf/requests'], check=True, capture_output=True
+        )
+        assert dependencies_in_sync([Requirement('requests@git+https://github.com/psf/requests')], venv.sys_path)
+
+
+@pytest.mark.requires_internet
+@pytest.mark.requires_git
+def test_dependency_git_revision(platform):
+    with TempVirtualEnv(sys.executable, platform) as venv:
+        platform.run_command(
+            ['pip', 'install', 'requests@git+https://github.com/psf/requests@main'], check=True, capture_output=True
+        )
+        assert dependencies_in_sync([Requirement('requests@git+https://github.com/psf/requests@main')], venv.sys_path)
+
+
+@pytest.mark.requires_internet
+@pytest.mark.requires_git
+def test_dependency_git_commit(platform):
+    with TempVirtualEnv(sys.executable, platform) as venv:
+        platform.run_command(
+            ['pip', 'install', 'requests@git+https://github.com/psf/requests@7f694b79e114c06fac5ec06019cada5a61e5570f'],
+            check=True,
+            capture_output=True,
+        )
+        assert dependencies_in_sync(
+            [Requirement('requests@git+https://github.com/psf/requests@7f694b79e114c06fac5ec06019cada5a61e5570f')],
+            venv.sys_path,
+        )
