@@ -1,15 +1,39 @@
+import os
+
 import pytest
 
 from hatch.project.core import Project
-from hatchling.utils.constants import DEFAULT_BUILD_SCRIPT
+from hatchling.utils.constants import DEFAULT_BUILD_SCRIPT, DEFAULT_CONFIG_FILE
 
 
-@pytest.fixture
-def local_builder(mock_backend_process, mocker):
-    if mock_backend_process:
-        mocker.patch('hatch.env.virtual.VirtualEnvironment.build_environment')
+class TestNoProject:
+    def test_random_directory(self, hatch, temp_dir, helpers):
+        with temp_dir.as_cwd():
+            result = hatch('version')
 
-    yield
+        assert result.exit_code == 1, result.output
+        assert result.output == helpers.dedent(
+            """
+            No project detected
+            """
+        )
+
+    def test_configured_project(self, hatch, temp_dir, helpers, config_file):
+        project = 'foo'
+        config_file.model.mode = 'project'
+        config_file.model.project = project
+        config_file.model.projects = {project: str(temp_dir)}
+        config_file.save()
+
+        with temp_dir.as_cwd():
+            result = hatch('version')
+
+        assert result.exit_code == 1, result.output
+        assert result.output == helpers.dedent(
+            """
+            Project foo (not a project)
+            """
+        )
 
 
 def test_incompatible_environment(hatch, temp_dir, helpers):
@@ -30,6 +54,19 @@ def test_incompatible_environment(hatch, temp_dir, helpers):
         project, 'default', {'skip-install': True, 'python': '9000', **project.config.envs['default']}
     )
 
+    build_script = path / DEFAULT_BUILD_SCRIPT
+    build_script.write_text(
+        helpers.dedent(
+            """
+            from hatchling.metadata.plugin.interface import MetadataHookInterface
+
+            class CustomMetadataHook(MetadataHookInterface):
+                def update(self, metadata):
+                    pass
+            """
+        )
+    )
+
     with path.as_cwd():
         result = hatch('version')
 
@@ -56,7 +93,7 @@ def test_show_dynamic(hatch, temp_dir):
     assert result.output == '0.0.1\n'
 
 
-@pytest.mark.usefixtures('local_builder')
+@pytest.mark.usefixtures('local_backend_process')
 def test_show_dynamic_missing_build_dependencies(hatch, helpers, temp_dir):
     project_name = 'My.App'
 
@@ -82,6 +119,44 @@ def test_show_dynamic_missing_build_dependencies(hatch, helpers, temp_dir):
     )
 
 
+@pytest.mark.usefixtures('local_backend_process')
+def test_plugin_dependencies_unmet(hatch, helpers, temp_dir, mock_plugin_installation):
+    project_name = 'My.App'
+
+    with temp_dir.as_cwd():
+        hatch('new', project_name)
+
+    path = temp_dir / 'my-app'
+
+    dependency = os.urandom(16).hex()
+    (path / DEFAULT_CONFIG_FILE).write_text(
+        helpers.dedent(
+            f"""
+            [env]
+            requires = ["{dependency}"]
+            """
+        )
+    )
+
+    project = Project(path)
+    config = dict(project.raw_config)
+    config['build-system']['requires'].append('foo')
+    project.save_config(config)
+
+    with path.as_cwd():
+        result = hatch('version')
+
+    assert result.exit_code == 0, result.output
+    assert result.output == helpers.dedent(
+        """
+        Syncing environment plugin requirements
+        Setting up build environment for missing dependencies
+        0.0.1
+        """
+    )
+    helpers.assert_plugin_installation(mock_plugin_installation, [dependency])
+
+
 def test_set_dynamic(hatch, helpers, temp_dir):
     project_name = 'My.App'
 
@@ -138,6 +213,7 @@ def test_show_static(hatch, temp_dir):
     config = dict(project.raw_config)
     config['project']['version'] = '1.2.3'
     config['project']['dynamic'].remove('version')
+    config['tool']['hatch']['metadata'] = {'hooks': {'foo': {}}}
     project.save_config(config)
 
     with path.as_cwd():
