@@ -366,11 +366,15 @@ def mock_plugin_installation(mocker):
     mocked_subprocess_run = mocker.MagicMock(returncode=0)
 
     def _mock(command, **kwargs):
-        if not isinstance(command, list) or command[:5] != [sys.executable, '-u', '-m', 'pip', 'install']:  # no cov
-            return subprocess_run(command, **kwargs)
+        if isinstance(command, list):
+            if command[:5] == [sys.executable, '-u', '-m', 'pip', 'install']:
+                mocked_subprocess_run(command, **kwargs)
+                return mocked_subprocess_run
 
-        mocked_subprocess_run(command, **kwargs)
-        return mocked_subprocess_run
+            if command[:3] == [sys.executable, 'self', 'python-path']:
+                return mocker.MagicMock(returncode=0, stdout=sys.executable.encode())
+
+        return subprocess_run(command, **kwargs)  # no cov
 
     mocker.patch('subprocess.run', side_effect=_mock)
 
